@@ -4,7 +4,7 @@ import warnings
 import numpy
 import numpy as np
 
-from .tensors import volume, create_ndarray_f32, create_ndarray_int64, get_attribute_data, is_valid_ndarray
+from .tensor import volume, create_ndarray_f32, create_ndarray_int64, get_attribute_data, is_valid_ndarray
 from .utils import NODEPROFILER_REGISTRY, tuple2str
 
 ADD_MACS = 1
@@ -684,6 +684,8 @@ class Scan(NodeBase):
 
     def infer_shape(self, intensors: [numpy.ndarray]):
         # TODO
+        if self.nboutput == 2:
+            return [intensors[3], intensors[3]]
         return [create_ndarray_f32((1, 1)), create_ndarray_f32((1, 1)), create_ndarray_f32((1,)),
                 intensors[3], intensors[3], ]
 
@@ -776,14 +778,14 @@ class Max(Min):
 
 
 @NODEPROFILER_REGISTRY.register()
-class Equal(NodeBase):
+class Equal(Min):
     def infer_shape(self, intensors: [numpy.ndarray]):
         result = numpy.equal(intensors[0], intensors[1])
         return [result]
 
 
 @NODEPROFILER_REGISTRY.register()
-class Greater(NodeBase):
+class Greater(Min):
     def infer_shape(self, intensors: [numpy.ndarray]):
         result = numpy.greater(intensors[0], intensors[1])
         return [result]
@@ -886,7 +888,10 @@ class ScatterElements(NodeBase):
 
 @NODEPROFILER_REGISTRY.register()
 class Hardmax(PWNBase):
-    pass
+    def __init__(self, n):
+        super(Hardmax, self).__init__(n)
+        self.ratio = 1
+        self.op_mac = CMP_MACS
 
 
 @NODEPROFILER_REGISTRY.register()
@@ -1285,7 +1290,7 @@ class ConstantOfShape(NodeBase):
 
     def infer_shape(self, intensors: [numpy.ndarray]):
         arr = numpy.zeros(intensors[0].astype(numpy.int64), dtype=numpy.float32)
-        if self.value is not None and len(self.value) == 1:
+        if self.value is not None:
             arr.fill(self.value[0])
         return [arr]
 
@@ -1301,6 +1306,8 @@ class Slice(FusedNode):
         super().__init__(nodeproto)
         attnames = ['axes', 'ends', 'starts']
         auto_add_attributes(nodeproto.attribute, attnames, self)
+        if hasattr(self, 'axes') and isinstance(self.axes, int):
+            self.axes = [self.axes]
 
     def infer_shape(self, intensors: [numpy.ndarray]):
         if len(intensors) == 3:
@@ -1462,6 +1469,7 @@ class InstanceNormalization(PWNBase):
     def __init__(self, node_proto):
         super().__init__(node_proto)
         self.op_mac = ADD_MACS + MUL_MACS + ADD_MACS + DIV_MACS
+        self.ratio = 1
 
 
 @NODEPROFILER_REGISTRY.register()
@@ -1503,11 +1511,7 @@ class Div(PWNBase):
 
 
 @NODEPROFILER_REGISTRY.register()
-class Range(PWNBase):
-    def __init__(self, node_proto):
-        super().__init__(node_proto)
-        self.op_mac = 1
-
+class Range(NodeBase):
     def infer_shape(self, intensors: [numpy.ndarray]):
         start = intensors[0]
         limit = intensors[1]
@@ -1516,17 +1520,28 @@ class Range(PWNBase):
 
 
 @NODEPROFILER_REGISTRY.register()
-class Floor(FusedNode):
+class Floor(PWNBase):
+    def __init__(self, n):
+        super().__init__(n)
+        self.ratio = 1
+        self.op_mac = CMP_MACS
+
     def infer_shape(self, intensors: [numpy.ndarray]):
         return [numpy.floor(intensors[0])]
 
 
 @NODEPROFILER_REGISTRY.register()
-class Ceil(FusedNode):
+class Ceil(PWNBase):
+    def __init__(self, n):
+        super().__init__(n)
+        self.ratio = 1
+        self.op_mac = CMP_MACS
+
     def infer_shape(self, intensors: [numpy.ndarray]):
         return [numpy.ceil(intensors[0])]
 
 
+
 @NODEPROFILER_REGISTRY.register()
 class Reciprocal(PWNBase):
     def __init__(self, node_proto):
@@ -1588,14 +1603,16 @@ class Einsum(NodeBase):
 class DequantizeLinear(PWNBase):
     def __init__(self, node_proto):
         super().__init__(node_proto)
-        self.op_mac = MUL_MACS
+        self.op_mac = MUL_MACS + ADD_MACS
+        self.ratio = 1
 
 
 @NODEPROFILER_REGISTRY.register()
 class QuantizeLinear(PWNBase):
     def __init__(self, node_proto):
         super().__init__(node_proto)
-        self.op_mac = MUL_MACS
+        self.op_mac = MUL_MACS + ADD_MACS
+        self.ratio = 1
 
 
 @NODEPROFILER_REGISTRY.register()
